	 
	  //
/* anotehn t* ontuh * / tnoahnt /* oo */
/* multi line...

ao */
/*  no escape \
oanot */
// escape nothing \o oeu
// escape newline \
still in a comment
// escape newline \

// a different comment
#version 430 core
in vec4 v;
void main() {}


