#extension GL_GOOGLE_cpp_style_line_directive : enable

__FILE__

#line 150 "a.h"
__FILE__

#line 24
__FILE__

#line 42
__FILE__

#line 30 "b.cc"
__FILE__

#line 10 3
__FILE__

#line 48
__FILE__

#line 4
__FILE__

#line 55 100
__FILE__

#line 1000 "c"
__FILE__

#line 42 1
__FILE__

#line 42 "this-is-a-quite-long-name-maybe-i-should-shorten-it"
__FILE__
